package MAPI;

use strict;
use warnings;

require Exporter;
require DynaLoader;

our @ISA = qw(Exporter DynaLoader);
our @EXPORT = qw();
our @EXPORT_OK = qw();
our %EXPORT_TAGS = qw();
our $VERSION = '1.00';

sub PPropFindProp($$)
{
    my ($propref, $ulPropTag) = @_;
    my $prop;
    
    foreach $prop (@$propref) {
        if($prop->{"ulPropTag"} == $ulPropTag) {
            return $prop;
        }
    }
    
    return;
}

sub PROP_TAG($$) {
    my ($type, $id) = @_;    
    return ($id << 16) | $type;
};

# Errors
use constant {
    MAPI_E_NOT_FOUND => 0x8004010f,
};

# Property types
use constant {
    PT_UNSPECIFIED 	=> 0x0000,
    PT_NULL 		=> 0x0001,
    PT_SHORT 		=> 0x0002,
    PT_LONG 		=> 0x0003,
    PT_FLOAT 		=> 0x0004,
    PT_DOUBLE 		=> 0x0005,
    PT_CURRENCY 	=> 0x0006,
    PT_APPTIME 		=> 0x0007,
    PT_ERROR 		=> 0x000A,
    PT_BOOLEAN 		=> 0x000B,
    PT_OBJECT 		=> 0x000D,
    PT_LONGLONG 	=> 0x0014,
    PT_STRING8 		=> 0x001E,
    PT_UNICODE 		=> 0x001F,
    PT_SYSTIME 		=> 0x0040,
    PT_CLSID 		=> 0x0048,
    PT_BINARY 		=> 0x0102,
    PT_MV_SHORT		=> 0x1002,
    PT_MV_LONG		=> 0x1003,
    PT_MV_FLOAT		=> 0x1004,
    PT_MV_DOUBLE	=> 0x1005,
    PT_MV_CURRENCY	=> 0x1006,
    PT_MV_APPTIME	=> 0x1007,
    PT_MV_BOOLEAN	=> 0x100B,
    PT_MV_LONGLONG	=> 0x1014,
    PT_MV_STRING8	=> 0x101E,
    PT_MV_UNICODE	=> 0x101F,
    PT_MV_SYSTIME	=> 0x1040,
    PT_MV_CLSID		=> 0x1048,
    PT_MV_BINARY	=> 0x1102,
};

# In UNICODE mode, switch to PT_UNICODE
use constant {
    PT_TSTRING		=> PT_STRING8,
    PT_MV_TSTRING	=> PT_MV_STRING8,
};

# Property aliases
use constant {
    PT_I2 => PT_SHORT,
    PT_I4 => PT_LONG,
    PT_R4 => PT_FLOAT,
    PT_R8 => PT_DOUBLE,
    PT_I8 => PT_LONGLONG
};
    
# Default properties
use constant {
    # Generated from
    # grep PR_ ../mapi4linux/include/mapitags.h | perl -e 'while(<>) { chomp(); if(/(PR_\S+).*(PROP_TAG\(.*\))/) { print "    " . $1 . " => " . $2 . ",\n"; }}'
    PR_ACKNOWLEDGEMENT_MODE => PROP_TAG( PT_LONG,      0x0001),
    PR_ALTERNATE_RECIPIENT_ALLOWED => PROP_TAG( PT_BOOLEAN,   0x0002),
    PR_AUTHORIZING_USERS => PROP_TAG( PT_BINARY,    0x0003),
    PR_AUTO_FORWARD_COMMENT => PROP_TAG( PT_TSTRING,   0x0004),
    PR_AUTO_FORWARD_COMMENT_W => PROP_TAG( PT_UNICODE,   0x0004),
    PR_AUTO_FORWARD_COMMENT_A => PROP_TAG( PT_STRING8,   0x0004),
    PR_AUTO_FORWARDED => PROP_TAG( PT_BOOLEAN,   0x0005),
    PR_CONTENT_CONFIDENTIALITY_ALGORITHM_ID => PROP_TAG( PT_BINARY,    0x0006),
    PR_CONTENT_CORRELATOR => PROP_TAG( PT_BINARY,    0x0007),
    PR_CONTENT_IDENTIFIER => PROP_TAG( PT_TSTRING,   0x0008),
    PR_CONTENT_IDENTIFIER_W => PROP_TAG( PT_UNICODE,   0x0008),
    PR_CONTENT_IDENTIFIER_A => PROP_TAG( PT_STRING8,   0x0008),
    PR_CONTENT_LENGTH => PROP_TAG( PT_LONG,      0x0009),
    PR_CONTENT_RETURN_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x000A),
    PR_CONVERSATION_KEY => PROP_TAG( PT_BINARY,    0x000B),
    PR_CONVERSION_EITS => PROP_TAG( PT_BINARY,    0x000C),
    PR_CONVERSION_WITH_LOSS_PROHIBITED => PROP_TAG( PT_BOOLEAN,   0x000D),
    PR_CONVERTED_EITS => PROP_TAG( PT_BINARY,    0x000E),
    PR_DEFERRED_DELIVERY_TIME => PROP_TAG( PT_SYSTIME,   0x000F),
    PR_DELIVER_TIME => PROP_TAG( PT_SYSTIME,   0x0010),
    PR_DISCARD_REASON => PROP_TAG( PT_LONG,      0x0011),
    PR_DISCLOSURE_OF_RECIPIENTS => PROP_TAG( PT_BOOLEAN,   0x0012),
    PR_DL_EXPANSION_HISTORY => PROP_TAG( PT_BINARY,    0x0013),
    PR_DL_EXPANSION_PROHIBITED => PROP_TAG( PT_BOOLEAN,   0x0014),
    PR_EXPIRY_TIME => PROP_TAG( PT_SYSTIME,   0x0015),
    PR_IMPLICIT_CONVERSION_PROHIBITED => PROP_TAG( PT_BOOLEAN,   0x0016),
    PR_IMPORTANCE => PROP_TAG( PT_LONG,      0x0017),
    PR_IPM_ID => PROP_TAG( PT_BINARY,    0x0018),
    PR_LATEST_DELIVERY_TIME => PROP_TAG( PT_SYSTIME,   0x0019),
    PR_MESSAGE_CLASS => PROP_TAG( PT_TSTRING,   0x001A),
    PR_MESSAGE_CLASS_W => PROP_TAG( PT_UNICODE,   0x001A),
    PR_MESSAGE_CLASS_A => PROP_TAG( PT_STRING8,   0x001A),
    PR_MESSAGE_DELIVERY_ID => PROP_TAG( PT_BINARY,    0x001B),
    PR_MESSAGE_SECURITY_LABEL => PROP_TAG( PT_BINARY,    0x001E),
    PR_OBSOLETED_IPMS => PROP_TAG( PT_BINARY,    0x001F),
    PR_ORIGINALLY_INTENDED_RECIPIENT_NAME => PROP_TAG( PT_BINARY,    0x0020),
    PR_ORIGINAL_EITS => PROP_TAG( PT_BINARY,    0x0021),
    PR_ORIGINATOR_CERTIFICATE => PROP_TAG( PT_BINARY,    0x0022),
    PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0023),
    PR_ORIGINATOR_RETURN_ADDRESS => PROP_TAG( PT_BINARY,    0x0024),
    PR_PARENT_KEY => PROP_TAG( PT_BINARY,    0x0025),
    PR_PRIORITY => PROP_TAG( PT_LONG,      0x0026),
    PR_ORIGIN_CHECK => PROP_TAG( PT_BINARY,    0x0027),
    PR_PROOF_OF_SUBMISSION_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0028),
    PR_READ_RECEIPT_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0029),
    PR_RECEIPT_TIME => PROP_TAG( PT_SYSTIME,   0x002A),
    PR_RECIPIENT_REASSIGNMENT_PROHIBITED => PROP_TAG( PT_BOOLEAN,   0x002B),
    PR_REDIRECTION_HISTORY => PROP_TAG( PT_BINARY,    0x002C),
    PR_RELATED_IPMS => PROP_TAG( PT_BINARY,    0x002D),
    PR_ORIGINAL_SENSITIVITY => PROP_TAG( PT_LONG,      0x002E),
    PR_LANGUAGES => PROP_TAG( PT_TSTRING,   0x002F),
    PR_LANGUAGES_W => PROP_TAG( PT_UNICODE,   0x002F),
    PR_LANGUAGES_A => PROP_TAG( PT_STRING8,   0x002F),
    PR_REPLY_TIME => PROP_TAG( PT_SYSTIME,   0x0030),
    PR_REPORT_TAG => PROP_TAG( PT_BINARY,    0x0031),
    PR_REPORT_TIME => PROP_TAG( PT_SYSTIME,   0x0032),
    PR_RETURNED_IPM => PROP_TAG( PT_BOOLEAN,   0x0033),
    PR_SECURITY => PROP_TAG( PT_LONG,      0x0034),
    PR_INCOMPLETE_COPY => PROP_TAG( PT_BOOLEAN,   0x0035),
    PR_SENSITIVITY => PROP_TAG( PT_LONG,      0x0036),
    PR_SUBJECT => PROP_TAG( PT_TSTRING,   0x0037),
    PR_SUBJECT_W => PROP_TAG( PT_UNICODE,   0x0037),
    PR_SUBJECT_A => PROP_TAG( PT_STRING8,   0x0037),
    PR_SUBJECT_IPM => PROP_TAG( PT_BINARY,    0x0038),
    PR_CLIENT_SUBMIT_TIME => PROP_TAG( PT_SYSTIME,   0x0039),
    PR_REPORT_NAME => PROP_TAG( PT_TSTRING,   0x003A),
    PR_REPORT_NAME_W => PROP_TAG( PT_UNICODE,   0x003A),
    PR_REPORT_NAME_A => PROP_TAG( PT_STRING8,   0x003A),
    PR_SENT_REPRESENTING_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x003B),
    PR_X400_CONTENT_TYPE => PROP_TAG( PT_BINARY,    0x003C),
    PR_SUBJECT_PREFIX => PROP_TAG( PT_TSTRING,   0x003D),
    PR_SUBJECT_PREFIX_W => PROP_TAG( PT_UNICODE,   0x003D),
    PR_SUBJECT_PREFIX_A => PROP_TAG( PT_STRING8,   0x003D),
    PR_NON_RECEIPT_REASON => PROP_TAG( PT_LONG,      0x003E),
    PR_RECEIVED_BY_ENTRYID => PROP_TAG( PT_BINARY,    0x003F),
    PR_RECEIVED_BY_NAME => PROP_TAG( PT_TSTRING,   0x0040),
    PR_RECEIVED_BY_NAME_W => PROP_TAG( PT_UNICODE,   0x0040),
    PR_RECEIVED_BY_NAME_A => PROP_TAG( PT_STRING8,   0x0040),
    PR_SENT_REPRESENTING_ENTRYID => PROP_TAG( PT_BINARY,    0x0041),
    PR_SENT_REPRESENTING_NAME => PROP_TAG( PT_TSTRING,   0x0042),
    PR_SENT_REPRESENTING_NAME_W => PROP_TAG( PT_UNICODE,   0x0042),
    PR_SENT_REPRESENTING_NAME_A => PROP_TAG( PT_STRING8,   0x0042),
    PR_RCVD_REPRESENTING_ENTRYID => PROP_TAG( PT_BINARY,    0x0043),
    PR_RCVD_REPRESENTING_NAME => PROP_TAG( PT_TSTRING,   0x0044),
    PR_RCVD_REPRESENTING_NAME_W => PROP_TAG( PT_UNICODE,   0x0044),
    PR_RCVD_REPRESENTING_NAME_A => PROP_TAG( PT_STRING8,   0x0044),
    PR_REPORT_ENTRYID => PROP_TAG( PT_BINARY,    0x0045),
    PR_READ_RECEIPT_ENTRYID => PROP_TAG( PT_BINARY,    0x0046),
    PR_MESSAGE_SUBMISSION_ID => PROP_TAG( PT_BINARY,    0x0047),
    PR_PROVIDER_SUBMIT_TIME => PROP_TAG( PT_SYSTIME,   0x0048),
    PR_ORIGINAL_SUBJECT => PROP_TAG( PT_TSTRING,   0x0049),
    PR_ORIGINAL_SUBJECT_W => PROP_TAG( PT_UNICODE,   0x0049),
    PR_ORIGINAL_SUBJECT_A => PROP_TAG( PT_STRING8,   0x0049),
    PR_DISC_VAL => PROP_TAG( PT_BOOLEAN,   0x004A),
    PR_ORIG_MESSAGE_CLASS => PROP_TAG( PT_TSTRING,   0x004B),
    PR_ORIG_MESSAGE_CLASS_W => PROP_TAG( PT_UNICODE,   0x004B),
    PR_ORIG_MESSAGE_CLASS_A => PROP_TAG( PT_STRING8,   0x004B),
    PR_ORIGINAL_AUTHOR_ENTRYID => PROP_TAG( PT_BINARY,    0x004C),
    PR_ORIGINAL_AUTHOR_NAME => PROP_TAG( PT_TSTRING,   0x004D),
    PR_ORIGINAL_AUTHOR_NAME_W => PROP_TAG( PT_UNICODE,   0x004D),
    PR_ORIGINAL_AUTHOR_NAME_A => PROP_TAG( PT_STRING8,   0x004D),
    PR_ORIGINAL_SUBMIT_TIME => PROP_TAG( PT_SYSTIME,   0x004E),
    PR_REPLY_RECIPIENT_ENTRIES => PROP_TAG( PT_BINARY,    0x004F),
    PR_REPLY_RECIPIENT_NAMES => PROP_TAG( PT_TSTRING,   0x0050),
    PR_REPLY_RECIPIENT_NAMES_W => PROP_TAG( PT_UNICODE,   0x0050),
    PR_REPLY_RECIPIENT_NAMES_A => PROP_TAG( PT_STRING8,   0x0050),
    PR_RECEIVED_BY_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x0051),
    PR_RCVD_REPRESENTING_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x0052),
    PR_READ_RECEIPT_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x0053),
    PR_REPORT_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x0054),
    PR_ORIGINAL_DELIVERY_TIME => PROP_TAG( PT_SYSTIME,   0x0055),
    PR_ORIGINAL_AUTHOR_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x0056),
    PR_MESSAGE_TO_ME => PROP_TAG( PT_BOOLEAN,   0x0057),
    PR_MESSAGE_CC_ME => PROP_TAG( PT_BOOLEAN,   0x0058),
    PR_MESSAGE_RECIP_ME => PROP_TAG( PT_BOOLEAN,   0x0059),
    PR_ORIGINAL_SENDER_NAME => PROP_TAG( PT_TSTRING,   0x005A),
    PR_ORIGINAL_SENDER_NAME_W => PROP_TAG( PT_UNICODE,   0x005A),
    PR_ORIGINAL_SENDER_NAME_A => PROP_TAG( PT_STRING8,   0x005A),
    PR_ORIGINAL_SENDER_ENTRYID => PROP_TAG( PT_BINARY,    0x005B),
    PR_ORIGINAL_SENDER_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x005C),
    PR_ORIGINAL_SENT_REPRESENTING_NAME => PROP_TAG( PT_TSTRING,   0x005D),
    PR_ORIGINAL_SENT_REPRESENTING_NAME_W => PROP_TAG( PT_UNICODE,   0x005D),
    PR_ORIGINAL_SENT_REPRESENTING_NAME_A => PROP_TAG( PT_STRING8,   0x005D),
    PR_ORIGINAL_SENT_REPRESENTING_ENTRYID => PROP_TAG( PT_BINARY,    0x005E),
    PR_ORIGINAL_SENT_REPRESENTING_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x005F),
    PR_START_DATE => PROP_TAG( PT_SYSTIME,   0x0060),
    PR_END_DATE => PROP_TAG( PT_SYSTIME,   0x0061),
    PR_OWNER_APPT_ID => PROP_TAG( PT_LONG,      0x0062),
    PR_RESPONSE_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0063),
    PR_SENT_REPRESENTING_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0064),
    PR_SENT_REPRESENTING_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0064),
    PR_SENT_REPRESENTING_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0064),
    PR_SENT_REPRESENTING_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x0065),
    PR_SENT_REPRESENTING_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x0065),
    PR_SENT_REPRESENTING_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x0065),
    PR_ORIGINAL_SENDER_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0066),
    PR_ORIGINAL_SENDER_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0066),
    PR_ORIGINAL_SENDER_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0066),
    PR_ORIGINAL_SENDER_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x0067),
    PR_ORIGINAL_SENDER_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x0067),
    PR_ORIGINAL_SENDER_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x0067),
    PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0068),
    PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0068),
    PR_ORIGINAL_SENT_REPRESENTING_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0068),
    PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x0069),
    PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x0069),
    PR_ORIGINAL_SENT_REPRESENTING_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x0069),
    PR_CONVERSATION_TOPIC => PROP_TAG( PT_TSTRING,   0x0070),
    PR_CONVERSATION_TOPIC_W => PROP_TAG( PT_UNICODE,   0x0070),
    PR_CONVERSATION_TOPIC_A => PROP_TAG( PT_STRING8,   0x0070),
    PR_CONVERSATION_INDEX => PROP_TAG( PT_BINARY,    0x0071),
    PR_ORIGINAL_DISPLAY_BCC => PROP_TAG( PT_TSTRING,   0x0072),
    PR_ORIGINAL_DISPLAY_BCC_W => PROP_TAG( PT_UNICODE,   0x0072),
    PR_ORIGINAL_DISPLAY_BCC_A => PROP_TAG( PT_STRING8,   0x0072),
    PR_ORIGINAL_DISPLAY_CC => PROP_TAG( PT_TSTRING,   0x0073),
    PR_ORIGINAL_DISPLAY_CC_W => PROP_TAG( PT_UNICODE,   0x0073),
    PR_ORIGINAL_DISPLAY_CC_A => PROP_TAG( PT_STRING8,   0x0073),
    PR_ORIGINAL_DISPLAY_TO => PROP_TAG( PT_TSTRING,   0x0074),
    PR_ORIGINAL_DISPLAY_TO_W => PROP_TAG( PT_UNICODE,   0x0074),
    PR_ORIGINAL_DISPLAY_TO_A => PROP_TAG( PT_STRING8,   0x0074),
    PR_RECEIVED_BY_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0075),
    PR_RECEIVED_BY_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0075),
    PR_RECEIVED_BY_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0075),
    PR_RECEIVED_BY_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x0076),
    PR_RECEIVED_BY_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x0076),
    PR_RECEIVED_BY_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x0076),
    PR_RCVD_REPRESENTING_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0077),
    PR_RCVD_REPRESENTING_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0077),
    PR_RCVD_REPRESENTING_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0077),
    PR_RCVD_REPRESENTING_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x0078),
    PR_RCVD_REPRESENTING_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x0078),
    PR_RCVD_REPRESENTING_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x0078),
    PR_ORIGINAL_AUTHOR_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0079),
    PR_ORIGINAL_AUTHOR_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0079),
    PR_ORIGINAL_AUTHOR_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0079),
    PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x007A),
    PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x007A),
    PR_ORIGINAL_AUTHOR_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x007A),
    PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x007B),
    PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x007B),
    PR_ORIGINALLY_INTENDED_RECIP_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x007B),
    PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x007C),
    PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x007C),
    PR_ORIGINALLY_INTENDED_RECIP_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x007C),
    PR_TRANSPORT_MESSAGE_HEADERS => PROP_TAG(PT_TSTRING,    0x007D),
    PR_TRANSPORT_MESSAGE_HEADERS_W => PROP_TAG(PT_UNICODE,    0x007D),
    PR_TRANSPORT_MESSAGE_HEADERS_A => PROP_TAG(PT_STRING8,    0x007D),
    PR_DELEGATION => PROP_TAG(PT_BINARY,     0x007E),
    PR_TNEF_CORRELATION_KEY => PROP_TAG(PT_BINARY,     0x007F),
    PR_BODY => PROP_TAG( PT_TSTRING,   0x1000),
    PR_BODY_W => PROP_TAG( PT_UNICODE,   0x1000),
    PR_BODY_A => PROP_TAG( PT_STRING8,   0x1000),
    PR_REPORT_TEXT => PROP_TAG( PT_TSTRING,   0x1001),
    PR_REPORT_TEXT_W => PROP_TAG( PT_UNICODE,   0x1001),
    PR_REPORT_TEXT_A => PROP_TAG( PT_STRING8,   0x1001),
    PR_ORIGINATOR_AND_DL_EXPANSION_HISTORY => PROP_TAG( PT_BINARY,    0x1002),
    PR_REPORTING_DL_NAME => PROP_TAG( PT_BINARY,    0x1003),
    PR_REPORTING_MTA_CERTIFICATE => PROP_TAG( PT_BINARY,    0x1004),
    PR_RTF_SYNC_BODY_CRC => PROP_TAG( PT_LONG,      0x1006),
    PR_RTF_SYNC_BODY_COUNT => PROP_TAG( PT_LONG,      0x1007),
    PR_RTF_SYNC_BODY_TAG => PROP_TAG( PT_TSTRING,   0x1008),
    PR_RTF_SYNC_BODY_TAG_W => PROP_TAG( PT_UNICODE,   0x1008),
    PR_RTF_SYNC_BODY_TAG_A => PROP_TAG( PT_STRING8,   0x1008),
    PR_RTF_COMPRESSED => PROP_TAG( PT_BINARY,    0x1009),
    PR_RTF_SYNC_PREFIX_COUNT => PROP_TAG( PT_LONG,      0x1010),
    PR_RTF_SYNC_TRAILING_COUNT => PROP_TAG( PT_LONG,      0x1011),
    PR_ORIGINALLY_INTENDED_RECIP_ENTRYID => PROP_TAG( PT_BINARY,    0x1012),
    PR_CONTENT_INTEGRITY_CHECK => PROP_TAG( PT_BINARY,    0x0C00),
    PR_EXPLICIT_CONVERSION => PROP_TAG( PT_LONG,      0x0C01),
    PR_IPM_RETURN_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0C02),
    PR_MESSAGE_TOKEN => PROP_TAG( PT_BINARY,    0x0C03),
    PR_NDR_REASON_CODE => PROP_TAG( PT_LONG,      0x0C04),
    PR_NDR_DIAG_CODE => PROP_TAG( PT_LONG,      0x0C05),
    PR_NON_RECEIPT_NOTIFICATION_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0C06),
    PR_DELIVERY_POINT => PROP_TAG( PT_LONG,      0x0C07),
    PR_ORIGINATOR_NON_DELIVERY_REPORT_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0C08),
    PR_ORIGINATOR_REQUESTED_ALTERNATE_RECIPIENT => PROP_TAG( PT_BINARY,    0x0C09),
    PR_PHYSICAL_DELIVERY_BUREAU_FAX_DELIVERY => PROP_TAG( PT_BOOLEAN,   0x0C0A),
    PR_PHYSICAL_DELIVERY_MODE => PROP_TAG( PT_LONG,      0x0C0B),
    PR_PHYSICAL_DELIVERY_REPORT_REQUEST => PROP_TAG( PT_LONG,      0x0C0C),
    PR_PHYSICAL_FORWARDING_ADDRESS => PROP_TAG( PT_BINARY,    0x0C0D),
    PR_PHYSICAL_FORWARDING_ADDRESS_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0C0E),
    PR_PHYSICAL_FORWARDING_PROHIBITED => PROP_TAG( PT_BOOLEAN,   0x0C0F),
    PR_PHYSICAL_RENDITION_ATTRIBUTES => PROP_TAG( PT_BINARY,    0x0C10),
    PR_PROOF_OF_DELIVERY => PROP_TAG( PT_BINARY,    0x0C11),
    PR_PROOF_OF_DELIVERY_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0C12),
    PR_RECIPIENT_CERTIFICATE => PROP_TAG( PT_BINARY,    0x0C13),
    PR_RECIPIENT_NUMBER_FOR_ADVICE => PROP_TAG( PT_TSTRING,   0x0C14),
    PR_RECIPIENT_NUMBER_FOR_ADVICE_W => PROP_TAG( PT_UNICODE,   0x0C14),
    PR_RECIPIENT_NUMBER_FOR_ADVICE_A => PROP_TAG( PT_STRING8,   0x0C14),
    PR_RECIPIENT_TYPE => PROP_TAG( PT_LONG,      0x0C15),
    PR_REGISTERED_MAIL_TYPE => PROP_TAG( PT_LONG,      0x0C16),
    PR_REPLY_REQUESTED => PROP_TAG( PT_BOOLEAN,   0x0C17),
    PR_REQUESTED_DELIVERY_METHOD => PROP_TAG( PT_LONG,      0x0C18),
    PR_SENDER_ENTRYID => PROP_TAG( PT_BINARY,    0x0C19),
    PR_SENDER_NAME => PROP_TAG( PT_TSTRING,   0x0C1A),
    PR_SENDER_NAME_W => PROP_TAG( PT_UNICODE,   0x0C1A),
    PR_SENDER_NAME_A => PROP_TAG( PT_STRING8,   0x0C1A),
    PR_SUPPLEMENTARY_INFO => PROP_TAG( PT_TSTRING,   0x0C1B),
    PR_SUPPLEMENTARY_INFO_W => PROP_TAG( PT_UNICODE,   0x0C1B),
    PR_SUPPLEMENTARY_INFO_A => PROP_TAG( PT_STRING8,   0x0C1B),
    PR_TYPE_OF_MTS_USER => PROP_TAG( PT_LONG,      0x0C1C),
    PR_SENDER_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x0C1D),
    PR_SENDER_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x0C1E),
    PR_SENDER_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x0C1E),
    PR_SENDER_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x0C1E),
    PR_SENDER_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x0C1F),
    PR_SENDER_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x0C1F),
    PR_SENDER_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x0C1F),
    PR_CURRENT_VERSION => PROP_TAG( PT_I8,        0x0E00),
    PR_DELETE_AFTER_SUBMIT => PROP_TAG( PT_BOOLEAN,   0x0E01),
    PR_DISPLAY_BCC => PROP_TAG( PT_TSTRING,   0x0E02),
    PR_DISPLAY_BCC_W => PROP_TAG( PT_UNICODE,   0x0E02),
    PR_DISPLAY_BCC_A => PROP_TAG( PT_STRING8,   0x0E02),
    PR_DISPLAY_CC => PROP_TAG( PT_TSTRING,   0x0E03),
    PR_DISPLAY_CC_W => PROP_TAG( PT_UNICODE,   0x0E03),
    PR_DISPLAY_CC_A => PROP_TAG( PT_STRING8,   0x0E03),
    PR_DISPLAY_TO => PROP_TAG( PT_TSTRING,   0x0E04),
    PR_DISPLAY_TO_W => PROP_TAG( PT_UNICODE,   0x0E04),
    PR_DISPLAY_TO_A => PROP_TAG( PT_STRING8,   0x0E04),
    PR_PARENT_DISPLAY => PROP_TAG( PT_TSTRING,   0x0E05),
    PR_PARENT_DISPLAY_W => PROP_TAG( PT_UNICODE,   0x0E05),
    PR_PARENT_DISPLAY_A => PROP_TAG( PT_STRING8,   0x0E05),
    PR_MESSAGE_DELIVERY_TIME => PROP_TAG( PT_SYSTIME,   0x0E06),
    PR_MESSAGE_FLAGS => PROP_TAG( PT_LONG,      0x0E07),
    PR_MESSAGE_SIZE => PROP_TAG( PT_LONG,      0x0E08),
    PR_PARENT_ENTRYID => PROP_TAG( PT_BINARY,    0x0E09),
    PR_SENTMAIL_ENTRYID => PROP_TAG( PT_BINARY,    0x0E0A),
    PR_CORRELATE => PROP_TAG( PT_BOOLEAN,   0x0E0C),
    PR_CORRELATE_MTSID => PROP_TAG( PT_BINARY,    0x0E0D),
    PR_DISCRETE_VALUES => PROP_TAG( PT_BOOLEAN,   0x0E0E),
    PR_RESPONSIBILITY => PROP_TAG( PT_BOOLEAN,   0x0E0F),
    PR_SPOOLER_STATUS => PROP_TAG( PT_LONG,      0x0E10),
    PR_TRANSPORT_STATUS => PROP_TAG( PT_LONG,      0x0E11),
    PR_MESSAGE_RECIPIENTS => PROP_TAG( PT_OBJECT,    0x0E12),
    PR_MESSAGE_ATTACHMENTS => PROP_TAG( PT_OBJECT,    0x0E13),
    PR_SUBMIT_FLAGS => PROP_TAG( PT_LONG,      0x0E14),
    PR_RECIPIENT_STATUS => PROP_TAG( PT_LONG,      0x0E15),
    PR_TRANSPORT_KEY => PROP_TAG( PT_LONG,      0x0E16),
    PR_MSG_STATUS => PROP_TAG( PT_LONG,      0x0E17),
    PR_MESSAGE_DOWNLOAD_TIME => PROP_TAG( PT_LONG,      0x0E18),
    PR_CREATION_VERSION => PROP_TAG( PT_I8,        0x0E19),
    PR_MODIFY_VERSION => PROP_TAG( PT_I8,        0x0E1A),
    PR_HASATTACH => PROP_TAG( PT_BOOLEAN,   0x0E1B),
    PR_BODY_CRC => PROP_TAG( PT_LONG,      0x0E1C),
    PR_NORMALIZED_SUBJECT => PROP_TAG( PT_TSTRING,   0x0E1D),
    PR_NORMALIZED_SUBJECT_W => PROP_TAG( PT_UNICODE,   0x0E1D),
    PR_NORMALIZED_SUBJECT_A => PROP_TAG( PT_STRING8,   0x0E1D),
    PR_RTF_IN_SYNC => PROP_TAG( PT_BOOLEAN,   0x0E1F),
    PR_ATTACH_SIZE => PROP_TAG( PT_LONG,      0x0E20),
    PR_ATTACH_NUM => PROP_TAG( PT_LONG,      0x0E21),
    PR_PREPROCESS => PROP_TAG( PT_BOOLEAN,   0x0E22),
    PR_ORIGINATING_MTA_CERTIFICATE => PROP_TAG( PT_BINARY,    0x0E25),
    PR_PROOF_OF_SUBMISSION => PROP_TAG( PT_BINARY,    0x0E26),
    PR_ENTRYID => PROP_TAG( PT_BINARY,    0x0FFF),
    PR_OBJECT_TYPE => PROP_TAG( PT_LONG,      0x0FFE),
    PR_ICON => PROP_TAG( PT_BINARY,    0x0FFD),
    PR_MINI_ICON => PROP_TAG( PT_BINARY,    0x0FFC),
    PR_STORE_ENTRYID => PROP_TAG( PT_BINARY,    0x0FFB),
    PR_STORE_RECORD_KEY => PROP_TAG( PT_BINARY,    0x0FFA),
    PR_RECORD_KEY => PROP_TAG( PT_BINARY,    0x0FF9),
    PR_MAPPING_SIGNATURE => PROP_TAG( PT_BINARY,    0x0FF8),
    PR_ACCESS_LEVEL => PROP_TAG( PT_LONG,      0x0FF7),
    PR_INSTANCE_KEY => PROP_TAG( PT_BINARY,    0x0FF6),
    PR_ROW_TYPE => PROP_TAG( PT_LONG,      0x0FF5),
    PR_ACCESS => PROP_TAG( PT_LONG,      0x0FF4),
    PR_ROWID => PROP_TAG( PT_LONG,      0x3000),
    PR_DISPLAY_NAME => PROP_TAG( PT_TSTRING,   0x3001),
    PR_DISPLAY_NAME_W => PROP_TAG( PT_UNICODE,   0x3001),
    PR_DISPLAY_NAME_A => PROP_TAG( PT_STRING8,   0x3001),
    PR_ADDRTYPE => PROP_TAG( PT_TSTRING,   0x3002),
    PR_ADDRTYPE_W => PROP_TAG( PT_UNICODE,   0x3002),
    PR_ADDRTYPE_A => PROP_TAG( PT_STRING8,   0x3002),
    PR_EMAIL_ADDRESS => PROP_TAG( PT_TSTRING,   0x3003),
    PR_EMAIL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x3003),
    PR_EMAIL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x3003),
    PR_COMMENT => PROP_TAG( PT_TSTRING,   0x3004),
    PR_COMMENT_W => PROP_TAG( PT_UNICODE,   0x3004),
    PR_COMMENT_A => PROP_TAG( PT_STRING8,   0x3004),
    PR_DEPTH => PROP_TAG( PT_LONG,      0x3005),
    PR_PROVIDER_DISPLAY => PROP_TAG( PT_TSTRING,   0x3006),
    PR_PROVIDER_DISPLAY_W => PROP_TAG( PT_UNICODE,   0x3006),
    PR_PROVIDER_DISPLAY_A => PROP_TAG( PT_STRING8,   0x3006),
    PR_CREATION_TIME => PROP_TAG( PT_SYSTIME,   0x3007),
    PR_LAST_MODIFICATION_TIME => PROP_TAG( PT_SYSTIME,   0x3008),
    PR_RESOURCE_FLAGS => PROP_TAG( PT_LONG,      0x3009),
    PR_PROVIDER_DLL_NAME => PROP_TAG( PT_TSTRING,   0x300A),
    PR_PROVIDER_DLL_NAME_W => PROP_TAG( PT_UNICODE,   0x300A),
    PR_PROVIDER_DLL_NAME_A => PROP_TAG( PT_STRING8,   0x300A),
    PR_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x300B),
    PR_PROVIDER_UID => PROP_TAG( PT_BINARY,    0x300C),
    PR_PROVIDER_ORDINAL => PROP_TAG( PT_LONG,      0x300D),
    PR_FORM_VERSION => PROP_TAG(PT_TSTRING,    0x3301),
    PR_FORM_VERSION_W => PROP_TAG(PT_UNICODE,    0x3301),
    PR_FORM_VERSION_A => PROP_TAG(PT_STRING8,    0x3301),
    PR_FORM_CLSID => PROP_TAG(PT_CLSID,      0x3302),
    PR_FORM_CONTACT_NAME => PROP_TAG(PT_TSTRING,    0x3303),
    PR_FORM_CONTACT_NAME_W => PROP_TAG(PT_UNICODE,    0x3303),
    PR_FORM_CONTACT_NAME_A => PROP_TAG(PT_STRING8,    0x3303),
    PR_FORM_CATEGORY => PROP_TAG(PT_TSTRING,    0x3304),
    PR_FORM_CATEGORY_W => PROP_TAG(PT_UNICODE,    0x3304),
    PR_FORM_CATEGORY_A => PROP_TAG(PT_STRING8,    0x3304),
    PR_FORM_CATEGORY_SUB => PROP_TAG(PT_TSTRING,    0x3305),
    PR_FORM_CATEGORY_SUB_W => PROP_TAG(PT_UNICODE,    0x3305),
    PR_FORM_CATEGORY_SUB_A => PROP_TAG(PT_STRING8,    0x3305),
    PR_FORM_HOST_MAP => PROP_TAG(PT_MV_LONG,    0x3306),
    PR_FORM_HIDDEN => PROP_TAG(PT_BOOLEAN,    0x3307),
    PR_FORM_DESIGNER_NAME => PROP_TAG(PT_TSTRING,    0x3308),
    PR_FORM_DESIGNER_NAME_W => PROP_TAG(PT_UNICODE,    0x3308),
    PR_FORM_DESIGNER_NAME_A => PROP_TAG(PT_STRING8,    0x3308),
    PR_FORM_DESIGNER_GUID => PROP_TAG(PT_CLSID,      0x3309),
    PR_FORM_MESSAGE_BEHAVIOR => PROP_TAG(PT_LONG,       0x330A),
    PR_DEFAULT_STORE => PROP_TAG( PT_BOOLEAN,   0x3400),
    PR_STORE_SUPPORT_MASK => PROP_TAG( PT_LONG,      0x340D),
    PR_STORE_STATE => PROP_TAG( PT_LONG,      0x340E),
    PR_IPM_SUBTREE_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x3410),
    PR_IPM_OUTBOX_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x3411),
    PR_IPM_WASTEBASKET_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x3412),
    PR_IPM_SENTMAIL_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x3413),
    PR_MDB_PROVIDER => PROP_TAG( PT_BINARY,    0x3414),
    PR_RECEIVE_FOLDER_SETTINGS => PROP_TAG( PT_OBJECT,    0x3415),
    PR_VALID_FOLDER_MASK => PROP_TAG( PT_LONG,      0x35DF),
    PR_IPM_SUBTREE_ENTRYID => PROP_TAG( PT_BINARY,    0x35E0),
    PR_IPM_OUTBOX_ENTRYID => PROP_TAG( PT_BINARY,    0x35E2),
    PR_IPM_WASTEBASKET_ENTRYID => PROP_TAG( PT_BINARY,    0x35E3),
    PR_IPM_SENTMAIL_ENTRYID => PROP_TAG( PT_BINARY,    0x35E4),
    PR_VIEWS_ENTRYID => PROP_TAG( PT_BINARY,    0x35E5),
    PR_COMMON_VIEWS_ENTRYID => PROP_TAG( PT_BINARY,    0x35E6),
    PR_FINDER_ENTRYID => PROP_TAG( PT_BINARY,    0x35E7),
    PR_CONTAINER_FLAGS => PROP_TAG( PT_LONG,      0x3600),
    PR_FOLDER_TYPE => PROP_TAG( PT_LONG,      0x3601),
    PR_CONTENT_COUNT => PROP_TAG( PT_LONG,      0x3602),
    PR_CONTENT_UNREAD => PROP_TAG( PT_LONG,      0x3603),
    PR_CREATE_TEMPLATES => PROP_TAG( PT_OBJECT,    0x3604),
    PR_DETAILS_TABLE => PROP_TAG( PT_OBJECT,    0x3605),
    PR_SEARCH => PROP_TAG( PT_OBJECT,    0x3607),
    PR_SELECTABLE => PROP_TAG( PT_BOOLEAN,   0x3609),
    PR_SUBFOLDERS => PROP_TAG( PT_BOOLEAN,   0x360A),
    PR_STATUS => PROP_TAG( PT_LONG,      0x360B),
    PR_ANR => PROP_TAG( PT_TSTRING,   0x360C),
    PR_ANR_W => PROP_TAG( PT_UNICODE,   0x360C),
    PR_ANR_A => PROP_TAG( PT_STRING8,   0x360C),
    PR_CONTENTS_SORT_ORDER => PROP_TAG( PT_MV_LONG,   0x360D),
    PR_CONTAINER_HIERARCHY => PROP_TAG( PT_OBJECT,    0x360E),
    PR_CONTAINER_CONTENTS => PROP_TAG( PT_OBJECT,    0x360F),
    PR_FOLDER_ASSOCIATED_CONTENTS => PROP_TAG( PT_OBJECT,    0x3610),
    PR_DEF_CREATE_DL => PROP_TAG( PT_BINARY,    0x3611),
    PR_DEF_CREATE_MAILUSER => PROP_TAG( PT_BINARY,    0x3612),
    PR_CONTAINER_CLASS => PROP_TAG( PT_TSTRING,   0x3613),
    PR_CONTAINER_CLASS_W => PROP_TAG( PT_UNICODE,   0x3613),
    PR_CONTAINER_CLASS_A => PROP_TAG( PT_STRING8,   0x3613),
    PR_CONTAINER_MODIFY_VERSION => PROP_TAG( PT_I8,        0x3614),
    PR_AB_PROVIDER_ID => PROP_TAG( PT_BINARY,    0x3615),
    PR_DEFAULT_VIEW_ENTRYID => PROP_TAG( PT_BINARY,    0x3616),
    PR_ASSOC_CONTENT_COUNT => PROP_TAG( PT_LONG,      0x3617),
    PR_ATTACHMENT_X400_PARAMETERS => PROP_TAG( PT_BINARY,    0x3700),
    PR_ATTACH_DATA_OBJ => PROP_TAG( PT_OBJECT,    0x3701),
    PR_ATTACH_DATA_BIN => PROP_TAG( PT_BINARY,    0x3701),
    PR_ATTACH_ENCODING => PROP_TAG( PT_BINARY,    0x3702),
    PR_ATTACH_EXTENSION => PROP_TAG( PT_TSTRING,   0x3703),
    PR_ATTACH_EXTENSION_W => PROP_TAG( PT_UNICODE,   0x3703),
    PR_ATTACH_EXTENSION_A => PROP_TAG( PT_STRING8,   0x3703),
    PR_ATTACH_FILENAME => PROP_TAG( PT_TSTRING,   0x3704),
    PR_ATTACH_FILENAME_W => PROP_TAG( PT_UNICODE,   0x3704),
    PR_ATTACH_FILENAME_A => PROP_TAG( PT_STRING8,   0x3704),
    PR_ATTACH_METHOD => PROP_TAG( PT_LONG,      0x3705),
    PR_ATTACH_LONG_FILENAME => PROP_TAG( PT_TSTRING,   0x3707),
    PR_ATTACH_LONG_FILENAME_W => PROP_TAG( PT_UNICODE,   0x3707),
    PR_ATTACH_LONG_FILENAME_A => PROP_TAG( PT_STRING8,   0x3707),
    PR_ATTACH_PATHNAME => PROP_TAG( PT_TSTRING,   0x3708),
    PR_ATTACH_PATHNAME_W => PROP_TAG( PT_UNICODE,   0x3708),
    PR_ATTACH_PATHNAME_A => PROP_TAG( PT_STRING8,   0x3708),
    PR_ATTACH_RENDERING => PROP_TAG( PT_BINARY,    0x3709),
    PR_ATTACH_TAG => PROP_TAG( PT_BINARY,    0x370A),
    PR_RENDERING_POSITION => PROP_TAG( PT_LONG,      0x370B),
    PR_ATTACH_TRANSPORT_NAME => PROP_TAG( PT_TSTRING,   0x370C),
    PR_ATTACH_TRANSPORT_NAME_W => PROP_TAG( PT_UNICODE,   0x370C),
    PR_ATTACH_TRANSPORT_NAME_A => PROP_TAG( PT_STRING8,   0x370C),
    PR_ATTACH_LONG_PATHNAME => PROP_TAG( PT_TSTRING,   0x370D),
    PR_ATTACH_LONG_PATHNAME_W => PROP_TAG( PT_UNICODE,   0x370D),
    PR_ATTACH_LONG_PATHNAME_A => PROP_TAG( PT_STRING8,   0x370D),
    PR_ATTACH_MIME_TAG => PROP_TAG( PT_TSTRING,   0x370E),
    PR_ATTACH_MIME_TAG_W => PROP_TAG( PT_UNICODE,   0x370E),
    PR_ATTACH_MIME_TAG_A => PROP_TAG( PT_STRING8,   0x370E),
    PR_ATTACH_ADDITIONAL_INFO => PROP_TAG( PT_BINARY,    0x370F),
    PR_DISPLAY_TYPE => PROP_TAG( PT_LONG,      0x3900),
    PR_TEMPLATEID => PROP_TAG( PT_BINARY,    0x3902),
    PR_PRIMARY_CAPABILITY => PROP_TAG( PT_BINARY,    0x3904),
    PR_7BIT_DISPLAY_NAME => PROP_TAG( PT_STRING8,   0x39FF),
    PR_ACCOUNT => PROP_TAG( PT_TSTRING,   0x3A00),
    PR_ACCOUNT_W => PROP_TAG( PT_UNICODE,   0x3A00),
    PR_ACCOUNT_A => PROP_TAG( PT_STRING8,   0x3A00),
    PR_ALTERNATE_RECIPIENT => PROP_TAG( PT_BINARY,    0x3A01),
    PR_CALLBACK_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A02),
    PR_CALLBACK_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A02),
    PR_CALLBACK_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A02),
    PR_CONVERSION_PROHIBITED => PROP_TAG( PT_BOOLEAN,   0x3A03),
    PR_DISCLOSE_RECIPIENTS => PROP_TAG( PT_BOOLEAN,   0x3A04),
    PR_GENERATION => PROP_TAG( PT_TSTRING,   0x3A05),
    PR_GENERATION_W => PROP_TAG( PT_UNICODE,   0x3A05),
    PR_GENERATION_A => PROP_TAG( PT_STRING8,   0x3A05),
    PR_GIVEN_NAME => PROP_TAG( PT_TSTRING,   0x3A06),
    PR_GIVEN_NAME_W => PROP_TAG( PT_UNICODE,   0x3A06),
    PR_GIVEN_NAME_A => PROP_TAG( PT_STRING8,   0x3A06),
    PR_GOVERNMENT_ID_NUMBER => PROP_TAG( PT_TSTRING,   0x3A07),
    PR_GOVERNMENT_ID_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A07),
    PR_GOVERNMENT_ID_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A07),
    PR_BUSINESS_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A08),
    PR_BUSINESS_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A08),
    PR_BUSINESS_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A08),
    PR_HOME_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A09),
    PR_HOME_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A09),
    PR_HOME_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A09),
    PR_INITIALS => PROP_TAG( PT_TSTRING,   0x3A0A),
    PR_INITIALS_W => PROP_TAG( PT_UNICODE,   0x3A0A),
    PR_INITIALS_A => PROP_TAG( PT_STRING8,   0x3A0A),
    PR_KEYWORD => PROP_TAG( PT_TSTRING,   0x3A0B),
    PR_KEYWORD_W => PROP_TAG( PT_UNICODE,   0x3A0B),
    PR_KEYWORD_A => PROP_TAG( PT_STRING8,   0x3A0B),
    PR_LANGUAGE => PROP_TAG( PT_TSTRING,   0x3A0C),
    PR_LANGUAGE_W => PROP_TAG( PT_UNICODE,   0x3A0C),
    PR_LANGUAGE_A => PROP_TAG( PT_STRING8,   0x3A0C),
    PR_LOCATION => PROP_TAG( PT_TSTRING,   0x3A0D),
    PR_LOCATION_W => PROP_TAG( PT_UNICODE,   0x3A0D),
    PR_LOCATION_A => PROP_TAG( PT_STRING8,   0x3A0D),
    PR_MAIL_PERMISSION => PROP_TAG( PT_BOOLEAN,   0x3A0E),
    PR_MHS_COMMON_NAME => PROP_TAG( PT_TSTRING,   0x3A0F),
    PR_MHS_COMMON_NAME_W => PROP_TAG( PT_UNICODE,   0x3A0F),
    PR_MHS_COMMON_NAME_A => PROP_TAG( PT_STRING8,   0x3A0F),
    PR_ORGANIZATIONAL_ID_NUMBER => PROP_TAG( PT_TSTRING,   0x3A10),
    PR_ORGANIZATIONAL_ID_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A10),
    PR_ORGANIZATIONAL_ID_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A10),
    PR_SURNAME => PROP_TAG( PT_TSTRING,   0x3A11),
    PR_SURNAME_W => PROP_TAG( PT_UNICODE,   0x3A11),
    PR_SURNAME_A => PROP_TAG( PT_STRING8,   0x3A11),
    PR_ORIGINAL_ENTRYID => PROP_TAG( PT_BINARY,    0x3A12),
    PR_ORIGINAL_DISPLAY_NAME => PROP_TAG( PT_TSTRING,   0x3A13),
    PR_ORIGINAL_DISPLAY_NAME_W => PROP_TAG( PT_UNICODE,   0x3A13),
    PR_ORIGINAL_DISPLAY_NAME_A => PROP_TAG( PT_STRING8,   0x3A13),
    PR_ORIGINAL_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x3A14),
    PR_POSTAL_ADDRESS => PROP_TAG( PT_TSTRING,   0x3A15),
    PR_POSTAL_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x3A15),
    PR_POSTAL_ADDRESS_A => PROP_TAG( PT_STRING8,   0x3A15),
    PR_COMPANY_NAME => PROP_TAG( PT_TSTRING,   0x3A16),
    PR_COMPANY_NAME_W => PROP_TAG( PT_UNICODE,   0x3A16),
    PR_COMPANY_NAME_A => PROP_TAG( PT_STRING8,   0x3A16),
    PR_TITLE => PROP_TAG( PT_TSTRING,   0x3A17),
    PR_TITLE_W => PROP_TAG( PT_UNICODE,   0x3A17),
    PR_TITLE_A => PROP_TAG( PT_STRING8,   0x3A17),
    PR_DEPARTMENT_NAME => PROP_TAG( PT_TSTRING,   0x3A18),
    PR_DEPARTMENT_NAME_W => PROP_TAG( PT_UNICODE,   0x3A18),
    PR_DEPARTMENT_NAME_A => PROP_TAG( PT_STRING8,   0x3A18),
    PR_OFFICE_LOCATION => PROP_TAG( PT_TSTRING,   0x3A19),
    PR_OFFICE_LOCATION_W => PROP_TAG( PT_UNICODE,   0x3A19),
    PR_OFFICE_LOCATION_A => PROP_TAG( PT_STRING8,   0x3A19),
    PR_PRIMARY_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A1A),
    PR_PRIMARY_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A1A),
    PR_PRIMARY_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A1A),
    PR_BUSINESS2_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A1B),
    PR_BUSINESS2_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A1B),
    PR_BUSINESS2_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A1B),
    PR_MOBILE_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A1C),
    PR_MOBILE_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A1C),
    PR_MOBILE_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A1C),
    PR_RADIO_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A1D),
    PR_RADIO_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A1D),
    PR_RADIO_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A1D),
    PR_CAR_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A1E),
    PR_CAR_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A1E),
    PR_CAR_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A1E),
    PR_OTHER_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A1F),
    PR_OTHER_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A1F),
    PR_OTHER_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A1F),
    PR_TRANSMITABLE_DISPLAY_NAME => PROP_TAG( PT_TSTRING,   0x3A20),
    PR_TRANSMITABLE_DISPLAY_NAME_W => PROP_TAG( PT_UNICODE,   0x3A20),
    PR_TRANSMITABLE_DISPLAY_NAME_A => PROP_TAG( PT_STRING8,   0x3A20),
    PR_PAGER_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A21),
    PR_PAGER_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A21),
    PR_PAGER_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A21),
    PR_USER_CERTIFICATE => PROP_TAG( PT_BINARY,    0x3A22),
    PR_PRIMARY_FAX_NUMBER => PROP_TAG( PT_TSTRING,   0x3A23),
    PR_PRIMARY_FAX_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A23),
    PR_PRIMARY_FAX_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A23),
    PR_BUSINESS_FAX_NUMBER => PROP_TAG( PT_TSTRING,   0x3A24),
    PR_BUSINESS_FAX_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A24),
    PR_BUSINESS_FAX_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A24),
    PR_HOME_FAX_NUMBER => PROP_TAG( PT_TSTRING,   0x3A25),
    PR_HOME_FAX_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A25),
    PR_HOME_FAX_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A25),
    PR_COUNTRY => PROP_TAG( PT_TSTRING,   0x3A26),
    PR_COUNTRY_W => PROP_TAG( PT_UNICODE,   0x3A26),
    PR_COUNTRY_A => PROP_TAG( PT_STRING8,   0x3A26),
    PR_LOCALITY => PROP_TAG( PT_TSTRING,   0x3A27),
    PR_LOCALITY_W => PROP_TAG( PT_UNICODE,   0x3A27),
    PR_LOCALITY_A => PROP_TAG( PT_STRING8,   0x3A27),
    PR_STATE_OR_PROVINCE => PROP_TAG( PT_TSTRING,   0x3A28),
    PR_STATE_OR_PROVINCE_W => PROP_TAG( PT_UNICODE,   0x3A28),
    PR_STATE_OR_PROVINCE_A => PROP_TAG( PT_STRING8,   0x3A28),
    PR_STREET_ADDRESS => PROP_TAG( PT_TSTRING,   0x3A29),
    PR_STREET_ADDRESS_W => PROP_TAG( PT_UNICODE,   0x3A29),
    PR_STREET_ADDRESS_A => PROP_TAG( PT_STRING8,   0x3A29),
    PR_POSTAL_CODE => PROP_TAG( PT_TSTRING,   0x3A2A),
    PR_POSTAL_CODE_W => PROP_TAG( PT_UNICODE,   0x3A2A),
    PR_POSTAL_CODE_A => PROP_TAG( PT_STRING8,   0x3A2A),
    PR_POST_OFFICE_BOX => PROP_TAG( PT_TSTRING,   0x3A2B),
    PR_POST_OFFICE_BOX_W => PROP_TAG( PT_UNICODE,   0x3A2B),
    PR_POST_OFFICE_BOX_A => PROP_TAG( PT_STRING8,   0x3A2B),
    PR_TELEX_NUMBER => PROP_TAG( PT_TSTRING,   0x3A2C),
    PR_TELEX_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A2C),
    PR_TELEX_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A2C),
    PR_ISDN_NUMBER => PROP_TAG( PT_TSTRING,   0x3A2D),
    PR_ISDN_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A2D),
    PR_ISDN_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A2D),
    PR_ASSISTANT_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A2E),
    PR_ASSISTANT_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A2E),
    PR_ASSISTANT_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A2E),
    PR_HOME2_TELEPHONE_NUMBER => PROP_TAG( PT_TSTRING,   0x3A2F),
    PR_HOME2_TELEPHONE_NUMBER_W => PROP_TAG( PT_UNICODE,   0x3A2F),
    PR_HOME2_TELEPHONE_NUMBER_A => PROP_TAG( PT_STRING8,   0x3A2F),
    PR_ASSISTANT => PROP_TAG( PT_TSTRING,   0x3A30),
    PR_ASSISTANT_W => PROP_TAG( PT_UNICODE,   0x3A30),
    PR_ASSISTANT_A => PROP_TAG( PT_STRING8,   0x3A30),
    PR_SEND_RICH_INFO => PROP_TAG( PT_BOOLEAN,   0x3A40),
    PR_WEDDING_ANNIVERSARY => PROP_TAG( PT_SYSTIME, 0x3A41),
    PR_BIRTHDAY => PROP_TAG( PT_SYSTIME, 0x3A42),
    PR_HOBBIES => PROP_TAG( PT_TSTRING, 0x3A43),
    PR_HOBBIES_W => PROP_TAG( PT_UNICODE, 0x3A43),
    PR_HOBBIES_A => PROP_TAG( PT_STRING8, 0x3A43),
    PR_MIDDLE_NAME => PROP_TAG( PT_TSTRING, 0x3A44),
    PR_MIDDLE_NAME_W => PROP_TAG( PT_UNICODE, 0x3A44),
    PR_MIDDLE_NAME_A => PROP_TAG( PT_STRING8, 0x3A44),
    PR_DISPLAY_NAME_PREFIX => PROP_TAG( PT_TSTRING, 0x3A45),
    PR_DISPLAY_NAME_PREFIX_W => PROP_TAG( PT_UNICODE, 0x3A45),
    PR_DISPLAY_NAME_PREFIX_A => PROP_TAG( PT_STRING8, 0x3A45),
    PR_PROFESSION => PROP_TAG( PT_TSTRING, 0x3A46),
    PR_PROFESSION_W => PROP_TAG( PT_UNICODE, 0x3A46),
    PR_PROFESSION_A => PROP_TAG( PT_STRING8, 0x3A46),
    PR_PREFERRED_BY_NAME => PROP_TAG( PT_TSTRING, 0x3A47),
    PR_PREFERRED_BY_NAME_W => PROP_TAG( PT_UNICODE, 0x3A47),
    PR_PREFERRED_BY_NAME_A => PROP_TAG( PT_STRING8, 0x3A47),
    PR_SPOUSE_NAME => PROP_TAG( PT_TSTRING, 0x3A48),
    PR_SPOUSE_NAME_W => PROP_TAG( PT_UNICODE, 0x3A48),
    PR_SPOUSE_NAME_A => PROP_TAG( PT_STRING8, 0x3A48),
    PR_COMPUTER_NETWORK_NAME => PROP_TAG( PT_TSTRING, 0x3A49),
    PR_COMPUTER_NETWORK_NAME_W => PROP_TAG( PT_UNICODE, 0x3A49),
    PR_COMPUTER_NETWORK_NAME_A => PROP_TAG( PT_STRING8, 0x3A49),
    PR_CUSTOMER_ID => PROP_TAG( PT_TSTRING, 0x3A4A),
    PR_CUSTOMER_ID_W => PROP_TAG( PT_UNICODE, 0x3A4A),
    PR_CUSTOMER_ID_A => PROP_TAG( PT_STRING8, 0x3A4A),
    PR_TTYTDD_PHONE_NUMBER => PROP_TAG( PT_TSTRING, 0x3A4B),
    PR_TTYTDD_PHONE_NUMBER_W => PROP_TAG( PT_UNICODE, 0x3A4B),
    PR_TTYTDD_PHONE_NUMBER_A => PROP_TAG( PT_STRING8, 0x3A4B),
    PR_FTP_SITE => PROP_TAG( PT_TSTRING, 0x3A4C),
    PR_FTP_SITE_W => PROP_TAG( PT_UNICODE, 0x3A4C),
    PR_FTP_SITE_A => PROP_TAG( PT_STRING8, 0x3A4C),
    PR_GENDER => PROP_TAG( PT_SHORT, 0x3A4D),
    PR_MANAGER_NAME => PROP_TAG( PT_TSTRING, 0x3A4E),
    PR_MANAGER_NAME_W => PROP_TAG( PT_UNICODE, 0x3A4E),
    PR_MANAGER_NAME_A => PROP_TAG( PT_STRING8, 0x3A4E),
    PR_NICKNAME => PROP_TAG( PT_TSTRING, 0x3A4F),
    PR_NICKNAME_W => PROP_TAG( PT_UNICODE, 0x3A4F),
    PR_NICKNAME_A => PROP_TAG( PT_STRING8, 0x3A4F),
    PR_PERSONAL_HOME_PAGE => PROP_TAG( PT_TSTRING, 0x3A50),
    PR_PERSONAL_HOME_PAGE_W => PROP_TAG( PT_UNICODE, 0x3A50),
    PR_PERSONAL_HOME_PAGE_A => PROP_TAG( PT_STRING8, 0x3A50),
    PR_BUSINESS_HOME_PAGE => PROP_TAG( PT_TSTRING, 0x3A51),
    PR_BUSINESS_HOME_PAGE_W => PROP_TAG( PT_UNICODE, 0x3A51),
    PR_BUSINESS_HOME_PAGE_A => PROP_TAG( PT_STRING8, 0x3A51),
    PR_CONTACT_VERSION => PROP_TAG( PT_CLSID, 0x3A52),
    PR_CONTACT_ENTRYIDS => PROP_TAG( PT_MV_BINARY, 0x3A53),
    PR_CONTACT_ADDRTYPES => PROP_TAG( PT_MV_TSTRING, 0x3A54),
    PR_CONTACT_ADDRTYPES_W => PROP_TAG( PT_MV_UNICODE, 0x3A54),
    PR_CONTACT_ADDRTYPES_A => PROP_TAG( PT_MV_STRING8, 0x3A54),
    PR_CONTACT_DEFAULT_ADDRESS_INDEX => PROP_TAG( PT_LONG, 0x3A55),
    PR_CONTACT_EMAIL_ADDRESSES => PROP_TAG( PT_MV_TSTRING, 0x3A56),
    PR_CONTACT_EMAIL_ADDRESSES_W => PROP_TAG( PT_MV_UNICODE, 0x3A56),
    PR_CONTACT_EMAIL_ADDRESSES_A => PROP_TAG( PT_MV_STRING8, 0x3A56),
    PR_COMPANY_MAIN_PHONE_NUMBER => PROP_TAG( PT_TSTRING, 0x3A57),
    PR_COMPANY_MAIN_PHONE_NUMBER_W => PROP_TAG( PT_UNICODE, 0x3A57),
    PR_COMPANY_MAIN_PHONE_NUMBER_A => PROP_TAG( PT_STRING8, 0x3A57),
    PR_CHILDRENS_NAMES => PROP_TAG( PT_MV_TSTRING, 0x3A58),
    PR_CHILDRENS_NAMES_W => PROP_TAG( PT_MV_UNICODE, 0x3A58),
    PR_CHILDRENS_NAMES_A => PROP_TAG( PT_MV_STRING8, 0x3A58),
    PR_HOME_ADDRESS_CITY => PROP_TAG( PT_TSTRING, 0x3A59),
    PR_HOME_ADDRESS_CITY_W => PROP_TAG( PT_UNICODE, 0x3A59),
    PR_HOME_ADDRESS_CITY_A => PROP_TAG( PT_STRING8, 0x3A59),
    PR_HOME_ADDRESS_COUNTRY => PROP_TAG( PT_TSTRING, 0x3A5A),
    PR_HOME_ADDRESS_COUNTRY_W => PROP_TAG( PT_UNICODE, 0x3A5A),
    PR_HOME_ADDRESS_COUNTRY_A => PROP_TAG( PT_STRING8, 0x3A5A),
    PR_HOME_ADDRESS_POSTAL_CODE => PROP_TAG( PT_TSTRING, 0x3A5B),
    PR_HOME_ADDRESS_POSTAL_CODE_W => PROP_TAG( PT_UNICODE, 0x3A5B),
    PR_HOME_ADDRESS_POSTAL_CODE_A => PROP_TAG( PT_STRING8, 0x3A5B),
    PR_HOME_ADDRESS_STATE_OR_PROVINCE => PROP_TAG( PT_TSTRING, 0x3A5C),
    PR_HOME_ADDRESS_STATE_OR_PROVINCE_W => PROP_TAG( PT_UNICODE, 0x3A5C),
    PR_HOME_ADDRESS_STATE_OR_PROVINCE_A => PROP_TAG( PT_STRING8, 0x3A5C),
    PR_HOME_ADDRESS_STREET => PROP_TAG( PT_TSTRING, 0x3A5D),
    PR_HOME_ADDRESS_STREET_W => PROP_TAG( PT_UNICODE, 0x3A5D),
    PR_HOME_ADDRESS_STREET_A => PROP_TAG( PT_STRING8, 0x3A5D),
    PR_HOME_ADDRESS_POST_OFFICE_BOX => PROP_TAG( PT_TSTRING, 0x3A5E),
    PR_HOME_ADDRESS_POST_OFFICE_BOX_W => PROP_TAG( PT_UNICODE, 0x3A5E),
    PR_HOME_ADDRESS_POST_OFFICE_BOX_A => PROP_TAG( PT_STRING8, 0x3A5E),
    PR_OTHER_ADDRESS_CITY => PROP_TAG( PT_TSTRING, 0x3A5F),
    PR_OTHER_ADDRESS_CITY_W => PROP_TAG( PT_UNICODE, 0x3A5F),
    PR_OTHER_ADDRESS_CITY_A => PROP_TAG( PT_STRING8, 0x3A5F),
    PR_OTHER_ADDRESS_COUNTRY => PROP_TAG( PT_TSTRING, 0x3A60),
    PR_OTHER_ADDRESS_COUNTRY_W => PROP_TAG( PT_UNICODE, 0x3A60),
    PR_OTHER_ADDRESS_COUNTRY_A => PROP_TAG( PT_STRING8, 0x3A60),
    PR_OTHER_ADDRESS_POSTAL_CODE => PROP_TAG( PT_TSTRING, 0x3A61),
    PR_OTHER_ADDRESS_POSTAL_CODE_W => PROP_TAG( PT_UNICODE, 0x3A61),
    PR_OTHER_ADDRESS_POSTAL_CODE_A => PROP_TAG( PT_STRING8, 0x3A61),
    PR_OTHER_ADDRESS_STATE_OR_PROVINCE => PROP_TAG( PT_TSTRING, 0x3A62),
    PR_OTHER_ADDRESS_STATE_OR_PROVINCE_W => PROP_TAG( PT_UNICODE, 0x3A62),
    PR_OTHER_ADDRESS_STATE_OR_PROVINCE_A => PROP_TAG( PT_STRING8, 0x3A62),
    PR_OTHER_ADDRESS_STREET => PROP_TAG( PT_TSTRING, 0x3A63),
    PR_OTHER_ADDRESS_STREET_W => PROP_TAG( PT_UNICODE, 0x3A63),
    PR_OTHER_ADDRESS_STREET_A => PROP_TAG( PT_STRING8, 0x3A63),
    PR_OTHER_ADDRESS_POST_OFFICE_BOX => PROP_TAG( PT_TSTRING, 0x3A64),
    PR_OTHER_ADDRESS_POST_OFFICE_BOX_W => PROP_TAG( PT_UNICODE, 0x3A64),
    PR_OTHER_ADDRESS_POST_OFFICE_BOX_A => PROP_TAG( PT_STRING8, 0x3A64),
    PR_STORE_PROVIDERS => PROP_TAG( PT_BINARY,    0x3D00),
    PR_AB_PROVIDERS => PROP_TAG( PT_BINARY,    0x3D01),
    PR_TRANSPORT_PROVIDERS => PROP_TAG( PT_BINARY,    0x3D02),
    PR_DEFAULT_PROFILE => PROP_TAG( PT_BOOLEAN,   0x3D04),
    PR_AB_SEARCH_PATH => PROP_TAG( PT_MV_BINARY, 0x3D05),
    PR_AB_DEFAULT_DIR => PROP_TAG( PT_BINARY,    0x3D06),
    PR_AB_DEFAULT_PAB => PROP_TAG( PT_BINARY,    0x3D07),
    PR_FILTERING_HOOKS => PROP_TAG( PT_BINARY,    0x3D08),
    PR_SERVICE_NAME => PROP_TAG( PT_TSTRING,   0x3D09),
    PR_SERVICE_NAME_W => PROP_TAG( PT_UNICODE,   0x3D09),
    PR_SERVICE_NAME_A => PROP_TAG( PT_STRING8,   0x3D09),
    PR_SERVICE_DLL_NAME => PROP_TAG( PT_TSTRING,   0x3D0A),
    PR_SERVICE_DLL_NAME_W => PROP_TAG( PT_UNICODE,   0x3D0A),
    PR_SERVICE_DLL_NAME_A => PROP_TAG( PT_STRING8,   0x3D0A),
    PR_SERVICE_ENTRY_NAME => PROP_TAG( PT_STRING8,   0x3D0B),
    PR_SERVICE_UID => PROP_TAG( PT_BINARY,    0x3D0C),
    PR_SERVICE_EXTRA_UIDS => PROP_TAG( PT_BINARY,    0x3D0D),
    PR_SERVICES => PROP_TAG( PT_BINARY,    0x3D0E),
    PR_SERVICE_SUPPORT_FILES => PROP_TAG( PT_MV_TSTRING, 0x3D0F),
    PR_SERVICE_SUPPORT_FILES_W => PROP_TAG( PT_MV_UNICODE, 0x3D0F),
    PR_SERVICE_SUPPORT_FILES_A => PROP_TAG( PT_MV_STRING8, 0x3D0F),
    PR_SERVICE_DELETE_FILES => PROP_TAG( PT_MV_TSTRING, 0x3D10),
    PR_SERVICE_DELETE_FILES_W => PROP_TAG( PT_MV_UNICODE, 0x3D10),
    PR_SERVICE_DELETE_FILES_A => PROP_TAG( PT_MV_STRING8, 0x3D10),
    PR_AB_SEARCH_PATH_UPDATE => PROP_TAG( PT_BINARY,     0x3D11),
    PR_PROFILE_NAME => PROP_TAG( PT_TSTRING,   0x3D12),
    PR_PROFILE_NAME_A => PROP_TAG( PT_STRING8,   0x3D12),
    PR_PROFILE_NAME_W => PROP_TAG( PT_UNICODE,   0x3D12),
    PR_IDENTITY_DISPLAY => PROP_TAG( PT_TSTRING,   0x3E00),
    PR_IDENTITY_DISPLAY_W => PROP_TAG( PT_UNICODE,   0x3E00),
    PR_IDENTITY_DISPLAY_A => PROP_TAG( PT_STRING8,   0x3E00),
    PR_IDENTITY_ENTRYID => PROP_TAG( PT_BINARY,    0x3E01),
    PR_RESOURCE_METHODS => PROP_TAG( PT_LONG,      0x3E02),
    PR_RESOURCE_TYPE => PROP_TAG( PT_LONG,      0x3E03),
    PR_STATUS_CODE => PROP_TAG( PT_LONG,      0x3E04),
    PR_IDENTITY_SEARCH_KEY => PROP_TAG( PT_BINARY,    0x3E05),
    PR_OWN_STORE_ENTRYID => PROP_TAG( PT_BINARY,    0x3E06),
    PR_RESOURCE_PATH => PROP_TAG( PT_TSTRING,   0x3E07),
    PR_RESOURCE_PATH_W => PROP_TAG( PT_UNICODE,   0x3E07),
    PR_RESOURCE_PATH_A => PROP_TAG( PT_STRING8,   0x3E07),
    PR_STATUS_STRING => PROP_TAG( PT_TSTRING,   0x3E08),
    PR_STATUS_STRING_W => PROP_TAG( PT_UNICODE,   0x3E08),
    PR_STATUS_STRING_A => PROP_TAG( PT_STRING8,   0x3E08),
    PR_X400_DEFERRED_DELIVERY_CANCEL => PROP_TAG( PT_BOOLEAN,   0x3E09),
    PR_HEADER_FOLDER_ENTRYID => PROP_TAG( PT_BINARY,    0x3E0A),
    PR_REMOTE_PROGRESS => PROP_TAG( PT_LONG,      0x3E0B),
    PR_REMOTE_PROGRESS_TEXT => PROP_TAG( PT_TSTRING,   0x3E0C),
    PR_REMOTE_PROGRESS_TEXT_W => PROP_TAG( PT_UNICODE,   0x3E0C),
    PR_REMOTE_PROGRESS_TEXT_A => PROP_TAG( PT_STRING8,   0x3E0C),
    PR_REMOTE_VALIDATE_OK => PROP_TAG( PT_BOOLEAN,   0x3E0D),
    PR_CONTROL_FLAGS => PROP_TAG( PT_LONG,      0x3F00),
    PR_CONTROL_STRUCTURE => PROP_TAG( PT_BINARY,    0x3F01),
    PR_CONTROL_TYPE => PROP_TAG( PT_LONG,      0x3F02),
    PR_DELTAX => PROP_TAG( PT_LONG,      0x3F03),
    PR_DELTAY => PROP_TAG( PT_LONG,      0x3F04),
    PR_XPOS => PROP_TAG( PT_LONG,      0x3F05),
    PR_YPOS => PROP_TAG( PT_LONG,      0x3F06),
    PR_CONTROL_ID => PROP_TAG( PT_BINARY,    0x3F07),
    PR_INITIAL_DETAILS_PANE => PROP_TAG( PT_LONG,      0x3F08),
};

# Various flags
use constant {
    MDB_WRITE =>   0x00000004,
    MAPI_MODIFY => 0x00000001,
    MAPI_UNICODE => 0x80000000,
};

# Table constants
use constant {
    TABLE_SORT_ASCEND =>  0x00000000,
    TABLE_SORT_DESCEND => 0x00000001,
    TABLE_SORT_COMBINE => 0x00000002,
};

# Restriction constants
use constant {
    RES_AND 			=> 0x00000000,
    RES_OR				=> 0x00000001,
    RES_NOT				=> 0x00000002,
    RES_CONTENT			=> 0x00000003,
    RES_PROPERTY		=> 0x00000004,
    RES_COMPAREPROPS	=> 0x00000005,
    RES_BITMASK			=> 0x00000006,
    RES_SIZE			=> 0x00000007,
    RES_EXIST			=> 0x00000008,
    RES_SUBRESTRICTION	=> 0x00000009,
    RES_COMMENT			=> 0x0000000A,
};

use constant {
    RELOP_LT	=> 0,
    RELOP_LE	=> 1,
    RELOP_GT	=> 2,
    RELOP_GE	=> 3,
    RELOP_EQ	=> 4,
    RELOP_NE	=> 5,
    RELOP_RE	=> 6,
    
    BMR_EQZ		=> 0,
    BMR_NEZ		=> 1,
    
    FL_FULLSTRING 	=> 0x00000000,
    FL_SUBSTRING  	=> 0x00000001,
    FL_PREFIX		=> 0x00000002,
    
    FL_IGNORECASE		=> 0x00010000,
    FL_IGNORENONSPACE 	=> 0x00020000,
    FL_LOOSE			=> 0x00040000,
};

bootstrap MAPI $VERSION;

# Build our exported variables by looping through all symbols defined previously
my $sym;
foreach $sym (keys %MAPI::) {
    # Skip subpackage symbols (eg MAPI::Zarafa::)
    if($sym =~ /::$/) {
        next;
    }
    
    # Add all symbols as OK to export, and add them to the 'all' tag
    push @EXPORT_OK, $sym;
    push @{$EXPORT_TAGS{"all"}}, $sym;

    # Split the symbols into 'tags', 'restrictions' and 'flags'
    if($sym =~ /^PR_/) {
        push @{$EXPORT_TAGS{"tags"}}, $sym;
    }
    elsif($sym =~ /^RES_/) {
        push @{$EXPORT_TAGS{"restrictions"}}, $sym;
    }
    else {
        push @{$EXPORT_TAGS{"flags"}}, $sym;
    }
} 

package MAPI::Util;

sub CreateProfileAndConfigure($$$) {
    my ($profname, $service, $props) = @_;
    my $hr;
    
    TRY: {
        
        ($hr, my $profadmin) = MAPI::MAPIAdminProfiles(0);

        if($hr != 0) {
            last TRY;
        }

        $hr = $profadmin->CreateProfile($profname,"",0,0);

        if($hr != 0) {  
            last TRY;
        }

        ($hr, my $serviceadmin) = $profadmin->AdminServices($profname, "", 0, 0);

        if($hr != 0) {
            last TRY;
        }

        $hr = $serviceadmin->CreateMsgService($service, $service, 0, 0);

        if($hr != 0) {
            last TRY;
        }

        ($hr, my $msgservicetable) = $serviceadmin->GetMsgServiceTable(0);

        if($hr != 0) {
            last TRY;
        }

        ($hr, my $rows) = $msgservicetable->QueryRows(1, 0);

        if($hr != 0) {
            last TRY;
        }

        if(@$rows != 1) {
            $hr = MAPI::MAPI_E_NOT_FOUND;
            last TRY;
        }

        my $provider_uid = MAPI::PPropFindProp($rows->[0], MAPI::PR_SERVICE_UID);

        if(!defined($provider_uid)) {
            $hr = MAPI::MAPI_E_NOT_FOUND;
            last TRY;
        }
        
        $hr = $serviceadmin->ConfigureMsgService($provider_uid->{"Value"}, 0, 0, $props);

        if($hr != 0) {
            last TRY;
        }
    }
    
    return $hr;
}

package MAPI::Zarafa;


use constant {
    PR_EC_BASE 				=> 0x6700,
};

use constant {
    PR_EC_PATH 				=> MAPI::PROP_TAG(MAPI::PT_STRING8, PR_EC_BASE + 0x00),
    PR_EC_USERNAME 			=> MAPI::PROP_TAG(MAPI::PT_STRING8, PR_EC_BASE + 0x01),
    PR_EC_USERPASSWORD		=> MAPI::PROP_TAG(MAPI::PT_STRING8, PR_EC_BASE + 0x02),
    PR_EC_PORT 				=> MAPI::PROP_TAG(MAPI::PT_STRING8, PR_EC_BASE + 0x03),
    PR_EC_FLAGS				=> MAPI::PROP_TAG(MAPI::PT_STRING8, PR_EC_BASE + 0x04),
};

sub OpenECSession($$;$)
{
    my ($user, $pass, $server) = @_;
    my ($session, $hr);
    
    if(!defined($server)) {
        $server = "file:///var/run/zarafa";
    }
 
    my $props = [
        { "ulPropTag" => MAPI::Zarafa::PR_EC_PATH, "Value" => $server },
        { "ulPropTag" => MAPI::Zarafa::PR_EC_USERNAME, "Value" => $user },
        { "ulPropTag" => MAPI::Zarafa::PR_EC_USERPASSWORD, "Value" => $pass },
    ];
    
    TRY: {
        $hr = MAPI::Util::CreateProfileAndConfigure("prof", "ZARAFA6", $props);
    
        if($hr != 0) { last TRY; }
        
        ($hr, $session) = MAPI::MAPILogonEx(0, "prof", "", 0);
        
        if($hr != 0) { last TRY; }
    }

    return ($hr, $session);
}

# Define subclassing
package MAPI::IUnknown;
our @ISA = qw();

package MAPI::IMAPISession;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IProfAdmin;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IMsgServiceAdmin;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IMAPITable;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IAddrBook;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IMAPIProp;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IMAPIContainer;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IMAPIAdviseSink;
our @ISA = qw(MAPI::IUnknown);

package MAPI::IMsgStore;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IMAPIFolder;
our @ISA = qw(MAPI::IMAPIContainer);

package MAPI::IMessage;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IAttach;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IABContainer;
our @ISA = qw(MAPI::IMAPIContainer);

package MAPI::IMailUser;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IDistList;
our @ISA = qw(MAPI::IMAPIContainer);

package MAPI::IMAPIStatus;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IProfSect;
our @ISA = qw(MAPI::IMAPIProp);

package MAPI::IProviderAdmin;
our @ISA = qw(MAPI::IUnknown);

package MAPI::Util::MAPINotifSink;
our @ISA = qw(MAPI::IMAPIAdviseSink);

if(MAPI::MAPIInitialize() == 0) {
  1;
} else {
  0;
}

